use std::os::linux::raw::stat;
use actix_web::{error::ErrorInternalServerError, http::header::ACCEPT, web, HttpResponse, Responder};
use entity::node_group;
use sea_orm::{ActiveModelTrait, ActiveValue, EntityTrait, IntoActiveModel};
use serde::{Deserialize, Serialize};
use uuid::Uuid;
use crate::AppState;

#[derive(Deserialize)]
pub struct CreateGroupWithoutId {
    name: String,
}

#[derive(Deserialize)]
pub struct CreateLicense {
    name: String,
    coord_la: f64,
    coord_lo: f64,
    group: uuid::Uuid,
}

#[derive(Serialize)]
struct GroupWithNode {
    #[serde(flatten)]
    group: entity::node_group::Model,
    node: Vec<entity::node::Model>,
}

pub async fn get_nodes(state: web::Data<AppState>) -> actix_web::Result<impl Responder> {
    let db = &state.db;

    let result = node_group::Entity::find()
        .find_with_related(entity::prelude::Node)
        .all(db)
        .await
        .map_err(ErrorInternalServerError)?
        .into_iter()
        .map(|(group, node)| GroupWithNode { group, node })
        .collect::<Vec<_>>();

    Ok(web::Json(result))
}

pub async fn create_group(
    state: web::Data<AppState>,
    group: web::Json<CreateGroupWithoutId>,
) -> actix_web::Result<impl Responder> {
    let db = &state.db;

    let group = group.into_inner();

    let group = entity::node_group::ActiveModel {
        id: ActiveValue::NotSet,
        name: ActiveValue::Set(group.name),
    };

    let result = group.insert(db).await.map_err(ErrorInternalServerError)?;

    Ok(web::Json(result))
}

pub async fn create_node(
    state: web::Data<AppState>,
    node: web::Json<CreateLicense>,
) -> actix_web::Result<impl Responder> {
    let db = &state.db;

    let node = node.into_inner();

    println!("Checking group ID: {:?}", node.group);

    let group_exists = entity::node_group::Entity::find_by_id(node.group)
        .one(db)
        .await
        .map_err(ErrorInternalServerError)?
        .is_some();

    if !group_exists {
        return Err(ErrorInternalServerError("Group ID does not exist"));
    }

    let node = entity::node::ActiveModel {
        id: ActiveValue::NotSet,
        name: ActiveValue::Set(node.name),
        status: ActiveValue::NotSet,
        coord_la: ActiveValue::Set(node.coord_la),
        coord_lo: ActiveValue::Set(node.coord_lo),
        temperature: ActiveValue::NotSet,
        battery_minimum: ActiveValue::NotSet,
        battery_current: ActiveValue::NotSet,
        battery_maximum: ActiveValue::NotSet,
        voltage: ActiveValue::NotSet,
        uptime: ActiveValue::NotSet,
        group: ActiveValue::Set(node.group),
    };

    let result = node.insert(db).await.map_err(ErrorInternalServerError)?;

    Ok(web::Json(result))
}

pub async fn delete_node(
    state: web::Data<AppState>,
    path: web::Path<Uuid>,
) -> actix_web::Result<impl Responder> {
    let id = path.into_inner();

    let db = &state.db;

    entity::node::Entity::delete_by_id(id)
        .exec(db)
        .await
        .map_err(ErrorInternalServerError)?;

    Ok(HttpResponse::Ok().finish())
}